@@ -31,10 +31,12 @@
 #include <gconf/gconf-client.h>
 #include <libgnomeui/gnome-href.h>
 #include <string.h>
+
 #include "yelp-settings.h"
+#include "yelp-debug.h"
 
 #define KEY_GNOME_DIR           "/desktop/gnome/interface"
-#define KEY_GNOME_VARIABLE_FONT KEY_GNOME_DIR "/font_name"
+#define KEY_GNOME_VARIABLE_FONT KEY_GNOME_DIR "/document_font_name"
 #define KEY_GNOME_FIXED_FONT    KEY_GNOME_DIR "/monospace_font_name"
 #define KEY_GNOME_GTK_THEME     KEY_GNOME_DIR "/gtk_theme"
 
@@ -44,12 +46,6 @@
 #define KEY_YELP_VARIABLE_FONT  KEY_YELP_DIR "/variable_font"
 #define KEY_YELP_FIXED_FONT     KEY_YELP_DIR "/fixed_font"
 
-#ifdef YELP_DEBUG
-#define d(x) x
-#else
-#define d(x)
-#endif
-
 static const gchar * const color_params[YELP_NUM_COLORS] = {
     "yelp.color.fg",
     "yelp.color.bg",
@@ -350,11 +346,24 @@ gchar *
 yelp_settings_get_font (YelpFontType font)
 {
     gchar *key;
+    gchar *retval = NULL;
+    GError *err = NULL;
     gboolean use;
 
     use = gconf_client_get_bool (gconf_client,
 				 KEY_YELP_SYSTEM_FONTS,
-				 NULL);
+				 &err);
+
+    /* check for some kind of gconf error here; if error exists, then
+     * default to using system fonts since the yelp font settings might
+     * be messed up in gconf as well */
+    if (err != NULL) {
+	g_warning ("An error occurred getting the gconf value %s: %s\n",
+	           KEY_YELP_SYSTEM_FONTS, err->message);
+	g_error_free (err);
+	err = NULL;
+	use = FALSE;
+    }
 
     switch (font) {
     case YELP_FONT_VARIABLE:
@@ -367,7 +376,44 @@ yelp_settings_get_font (YelpFontType font)
 	g_assert_not_reached ();
     }
 
-    return gconf_client_get_string (gconf_client, key, NULL);
+    retval = gconf_client_get_string (gconf_client, key, &err);
+
+    if (err != NULL || retval == NULL) {
+	g_warning ("An error occurred getting the gconf value '%s'\n", key);
+	if (err) {
+	    g_error_free (err);
+	    err = NULL;
+	}
+
+	/* ok, if we were looking at the gnome system wide font setting
+	 * and it failed, then look at the yelp font setting - and vice
+	 * versa */
+	if (font == YELP_FONT_VARIABLE)
+	    key = (use ? KEY_YELP_VARIABLE_FONT : KEY_GNOME_VARIABLE_FONT);
+	else if (font == YELP_FONT_FIXED)
+	    key = (use ? KEY_YELP_FIXED_FONT : KEY_GNOME_FIXED_FONT);
+	else g_assert_not_reached ();
+
+	retval = gconf_client_get_string (gconf_client, key, &err);
+
+	/* if we still have an error, something seriously went wrong, and
+	 * well just hardcode the values */
+	if (err != NULL || retval == NULL) {
+	    g_warning ("An error occurred getting the gconf value '%s'\n", key);
+	    if (err) {
+		g_error_free (err);
+		err = NULL;
+	    }
+
+	    if (font == YELP_FONT_VARIABLE)
+		retval = g_strdup ("Sans 10");
+	    else if (font == YELP_FONT_FIXED)
+		retval = g_strdup ("Monospace 10");
+	    else g_assert_not_reached ();
+	}
+    }
+
+    return retval;
 }
 
 const gchar *
@@ -468,7 +514,7 @@ gconf_system_fonts_cb (GConfClient   *client,
 {
     gboolean use;
 
-    d (g_print ("gconf_system_fonts_cb\n"));
+    debug_print (DB_FUNCTION, "entering\n");
 
     use = gconf_value_get_bool (gconf_entry_get_value (entry));
 
@@ -491,7 +537,7 @@ gconf_font_cb (GConfClient   *client,
     gulong handler;
     const gchar *font;
 
-    d (g_print ("gconf_font_cb\n"));
+    debug_print (DB_FUNCTION, "entering\n");
 
     font = gconf_value_get_string (gconf_entry_get_value (entry));
 
@@ -540,7 +586,7 @@ gconf_use_caret_cb (GConfClient  *client,
 static void
 gtk_theme_changed (GtkSettings *settings, gpointer user_data)
 {
-    d (g_print ("gtk_theme_changed\n"));
+    debug_print (DB_FUNCTION, "entering\n");
 
     settings_update (YELP_SETTINGS_INFO_COLOR);
 }
@@ -562,7 +608,7 @@ settings_update (YelpSettingsType type)
     guint16    rval, gval, bval;
     gint i;
 
-    d (g_print ("settings_update\n"));
+    debug_print (DB_FUNCTION, "entering\n");
 
     if (type & YELP_SETTINGS_INFO_COLOR) {
 	style = gtk_rc_get_style_by_paths (gtk_settings,
