@@ -153,6 +153,10 @@ static void        tree_drag_data_get_cb          (GtkWidget         *widget,
 						   guint              info,
 						   guint32            time,
 						   YelpWindow        *window);
+static void        tree_row_expand_cb             (GtkTreeView       *view,
+						   GtkTreePath       *path,
+						   GtkTreeViewColumn *column,
+						   YelpWindow        *window);
 
 /** UIManager Callbacks **/
 static void    window_add_widget        (GtkUIManager *ui_manager,
@@ -1307,6 +1311,9 @@ window_populate (YelpWindow *window)
     g_signal_connect (selection, "changed",
 		      G_CALLBACK (tree_selection_changed_cb),
 		      window);
+    g_signal_connect (priv->side_sects, "row-activated",
+		      G_CALLBACK (tree_row_expand_cb),
+		      window);
     g_signal_connect (priv->side_sects, "drag_data_get",
 		      G_CALLBACK (tree_drag_data_get_cb),
 		      window);
@@ -1531,6 +1538,7 @@ window_do_load_pager (YelpWindow  *window,
     startnow = FALSE;
 
     state = yelp_pager_get_state (pager);
+    debug_print (DB_DEBUG, "pager state=%d\n", state);
     switch (state) {
     case YELP_PAGER_STATE_ERROR:
 	error = yelp_pager_get_error (pager);
@@ -2035,18 +2043,26 @@ pager_finish_cb (YelpPager   *pager,
 {
     GError *error = NULL;
     YelpWindow  *window = YELP_WINDOW (user_data);
+    const gchar *page_id;
+    
+    page_id = yelp_pager_resolve_frag (pager,
+				       window->priv->current_frag);
 
     debug_print (DB_FUNCTION, "entering\n");
+    
+    if (!page_id && window->priv->current_frag &&
+	strcmp (window->priv->current_frag, "")) {
 
-    window_disconnect (window);
+	window_disconnect (window);
 
-    g_set_error (&error, YELP_ERROR, YELP_ERROR_NO_PAGE,
-		 _("The section ‘%s’ does not exist in this document. "
-		   "If you were directed to this section from a Help "
-		   "button in an application, please report this to "
-		   "the maintainers of that application."),
-		 window->priv->current_frag);
-    window_error (window, error, TRUE);
+	g_set_error (&error, YELP_ERROR, YELP_ERROR_NO_PAGE,
+		     _("The section ‘%s’ does not exist in this document. "
+		       "If you were directed to this section from a Help "
+		       "button in an application, please report this to "
+		       "the maintainers of that application."),
+		     window->priv->current_frag);
+	window_error (window, error, TRUE);
+    }
 
     /* FIXME: Remove the URI from the history and go back */
 }
@@ -2201,6 +2217,19 @@ tree_drag_data_get_cb (GtkWidget         *widget,
     g_free (uri);
 }
 
+void
+tree_row_expand_cb (GtkTreeView *view, GtkTreePath *path,
+		    GtkTreeViewColumn *column, YelpWindow *window)
+{
+	if (gtk_tree_view_row_expanded (view, path)) {
+		gtk_tree_view_collapse_row (view, path);
+	} else {
+		gtk_tree_view_expand_to_path (view, path);
+	}
+
+}
+
+
 /** UIManager Callbacks *******************************************************/
 
 static void
